--[[ svn: $Id: Moolah.lua 60013 2008-02-02 15:52:06Z crashmstr $

Moolah
    ...because there it is nice to know how much you have

By crashmstr (wowzn@crashmstr.com)

Purpose:
To keep track of money that you have across characters, as well as who has what. At some point, line graphs or stacked bar charts will be one way to look at the data.

--]]

Moolah = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")

local options = { 
    type='group',
    args = {
		 update = {
			 type = 'execute',
			 name = 'Update',
			 desc = 'Update the money!',
			 func = 'UpdateTheMoney',
			 order = -1,
		 },
		 show = {
			 type = 'execute',
			 name = 'Show',
			 desc = 'Show me the money!',
			 func = 'ShowMeTheMoney',
			 order = -1,
		 },
		 showgui = {
			 type = 'execute',
			 name = 'ShowGUI',
			 desc = 'Show me the money (GUI version)!',
			 func = 'ShowMeTheMoneyGUI',
			 order = -1,
		 },
		 horde = {
			 type = 'execute',
			 name = 'Horde',
			 desc = 'Horde me the money!',
			 func = 'HordeMeTheMoney',
			 order = -1,
		 },
		 alliance = {
			 type = 'execute',
			 name = 'Alliance',
			 desc = 'Alliance me the money!',
			 func = 'AllianceMeTheMoney',
			 order = -1,
		 },
        debug = {
            type = 'toggle',
            name = 'Debug window',
            desc = 'Toggles the debug window',
            get = "IsDebug",
            set = "ToggleDebug",
        },
    },
}

Moolah:RegisterDB("MoolahDB", "MoolahDBPerChar")
Moolah:RegisterDefaults('profile', { debug = true, indexCount = 0, moneyEvents = {}, moneyChars = {}, moneyFaction = { Alliance = 0, Horde = 0, }, moneyAll = 0, } )

local moolahFrame

function Moolah:IsDebug() return self.db.profile.debug; end
function Moolah:ToggleDebug() self.db.profile.debug = not self.db.profile.debug; end

function Moolah:FormatMoney(copperToConvert)
	local money = {
		gold = floor(copperToConvert/10000),
		silver = mod(floor(copperToConvert/100), 100),
		copper = mod(copperToConvert,100),
	}
	return money.gold .. "g " .. money.silver .. "s " .. money.copper .. "c"
end

function Moolah:AllMoney()
	self:Print("All " .. self:FormatMoney(self.db.profile.moneyAll))
end

function Moolah:MoneyByFaction(faction)
	self:Print("<" .. faction .. ">" .. " " .. self:FormatMoney(self.db.profile.moneyFaction[faction]))
	for c,cd in pairs(self.db.profile.moneyChars) do
		if cd.tFaction == faction then
			self:Print(c .. "<" .. cd.tFaction .. ">" .. " " .. self:FormatMoney(cd.tMoney))
		end
	end
end

function Moolah:HordeMeTheMoney()
	self:AllMoney()
	self:MoneyByFaction("Horde")
end
	
function Moolah:AllianceMeTheMoney()
	self:AllMoney()
	self:MoneyByFaction("Alliance")
end
	
function Moolah:ShowMeTheMoney(...)
	self:AllMoney()
	self:MoneyByFaction("Horde")
	self:MoneyByFaction("Alliance")
end

function Moolah:UpdateTheMoney()
	self:PLAYER_MONEY()
	self:ShowMeTheMoney()
end

function createMoolahFrame()
	moolahFrame = CreateFrame("Frame", "MoolahFrame", UIParent, "DialogBoxFrame")
	moolahFrame:SetWidth(500)
	moolahFrame:SetHeight(400)
	moolahFrame:SetPoint("CENTER")
	moolahFrame:SetBackdrop({bgFile = [[Interface\DialogFrame\UI-DialogBox-Background]],edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],tile = true, tileSize = 16, edgeSize = 16,insets = { left = 5, right = 5, top = 5, bottom = 5 }})
	moolahFrame:SetBackdropColor(0,0,0,1)

	local text = moolahFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlightLarge")
	moolahFrame.title = text
	text:SetPoint("TOP", 0, -5)

	moolahFrame:Hide()

	moolahFrame.lefts = {}
	moolahFrame.rights = {}
	moolahFrame.textLefts = {}
	moolahFrame.textRights = {}
	function moolahFrame:Clear()
		self.title:SetText("")
		for i = 1, #self.lefts do
			self.lefts[i] = nil
			self.rights[i] = nil
		end
	end

	function moolahFrame:AddLine(left, right)
		moolahFrame.lefts[#moolahFrame.lefts+1] = left
		moolahFrame.rights[#moolahFrame.rights+1] = right
	end

	local moolahFrame_Show = moolahFrame.Show
	function moolahFrame:Show(...)
		local maxLeftWidth = 0
		local maxRightWidth = 0
		local textHeight = 0
		for i = 1, #self.lefts do
			if not self.textLefts[i] then
				local left = moolahFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
				self.textLefts[i] = left
				local right = moolahFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
				self.textRights[i] = right
				if i == 1 then
					left:SetPoint("TOPRIGHT", moolahFrame, "TOPLEFT", 75, -35)
				else
					left:SetPoint("TOPRIGHT", self.textLefts[i-1], "BOTTOMRIGHT", 0, -5)
				end
				right:SetPoint("LEFT", left, "RIGHT", 5, 0)
			end
			self.textLefts[i]:SetText(self.lefts[i] .. ":")
			self.textRights[i]:SetText(self.rights[i])
			local leftWidth = self.textLefts[i]:GetWidth()
			local rightWidth = self.textRights[i]:GetWidth()
			textHeight = self.textLefts[i]:GetHeight()
			if maxLeftWidth < leftWidth then
				maxLeftWidth = leftWidth
			end
			if maxRightWidth < rightWidth then
				maxRightWidth = rightWidth
			end
		end
		for i = #self.lefts+1, #self.textLefts do
			self.textLefts[i]:SetText('')
			self.textRights[i]:SetText('')
		end
		moolahFrame:SetWidth(175 + maxRightWidth + 20)
		moolahFrame:SetHeight(#self.lefts * (textHeight + 5) + 100)

		moolahFrame_Show(self, ...)
	end
	moolahFrame:Hide()

	createMoolahFrame = nil
end

function Moolah:ShowMeTheMoneyGUI(...)
	if createMoolahFrame then
		createMoolahFrame()
	end
	moolahFrame:Clear()
	moolahFrame.title:SetText("Moolah Baby!")
	
	moolahFrame.currentAddon = self

	moolahFrame:AddLine("All", self:FormatMoney(self.db.profile.moneyAll))
	local myname = UnitName('player')
	local myfaction = UnitFactionGroup('player')
	for i,f in ipairs({"Horde", "Alliance"}) do
		local star = ""
		if f == myfaction then
			star = "*"
		end
		moolahFrame:AddLine("<" .. f .. ">" .. star, self:FormatMoney(self.db.profile.moneyFaction[f]))
		star = ""
		
		for c,cd in pairs(self.db.profile.moneyChars) do
			if c == myname then
				star = "*"
			end
			if cd.tFaction == f then
				moolahFrame:AddLine(c .. star, self:FormatMoney(cd.tMoney))
			end
			star = ""
		end
	end

	moolahFrame:Show()
end

function Moolah:OnInitialize()
    -- Called when the addon is loaded
 
end

function Moolah:OnEnable()
	-- Called when the addon is enabled
	self:RegisterChatCommand("/moolah", "/moolah", options)
	self:RegisterEvent('PLAYER_MONEY')
	self:PLAYER_MONEY()
	self:Print("|cFFFF0000Moolah|r: ENABLED! (because it is nice to know how much you have...")
end

function Moolah:OnDisable()
    -- Called when the addon is disabled
end


function Moolah:PLAYER_MONEY()
	--check data version
	if self.db.profile.dataVersion == nil then
		self.db.profile.indexCount = 0
		self.db.profile.moneyEvents = {}
	end
	self.db.profile.dataVersion = 2

	-- Called whenever the player's money changes
	self.db.profile.indexCount = self.db.profile.indexCount + 1
	name = UnitName('player')
	faction = UnitFactionGroup('player')
	money = GetMoney()
	dateStamp = date("%Y%m%d%H%M%S", time())


	self:Print(name .. ' now has ' .. self:FormatMoney(money) .. ' (' .. dateStamp .. ')')

	self.db.profile.moneyChars[name] =  { tStamp = dateStamp, tName = name, tFaction = faction, tMoney = money, }
	self.db.profile.moneyEvents[self.db.profile.indexCount] = {}

	for i,m in pairs(self.db.profile.moneyChars) do
		--save all current character data into this event
		--that way, for each event we have all known data
		self.db.profile.moneyEvents[self.db.profile.indexCount][i] = m
		if i == name then
			self.db.profile.moneyEvents[self.db.profile.indexCount][i].tMyEvent = true
		end
	end


	self.db.profile.moneyAll = 0
	self.db.profile.moneyFaction["Alliance"] = 0
	self.db.profile.moneyFaction["Horde"] = 0

	for i,m in pairs(self.db.profile.moneyChars) do
		self.db.profile.moneyAll = m.tMoney + self.db.profile.moneyAll
		self.db.profile.moneyFaction[m.tFaction] = m.tMoney + self.db.profile.moneyFaction[m.tFaction]
	end
end
